program vectorization_test

implicit none
integer, parameter :: myk=kind(1.d0)
integer, parameter :: n=1000
integer, parameter :: n_short=4
real(myk) :: a(n),b(n),c(n),d(n),double
real(myk) :: da(n,n),db(n,n),dc(n,n)
real(myk) :: summ,x,w
integer :: ai(n),ind
integer :: i,j,whatis

call random_number(a)
call random_number(b)
call random_number(c)
call random_number(d)
call random_number(da)
call random_number(db)
call random_number(dc)

ai = int(100._myk*a)

print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 1 - simple vector add
!-----------------------------------------------------------------
do i=1,n
   c(i) = a(i)+b(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 2 - short vector add
!-----------------------------------------------------------------
do i=1,n_short
   c(i) = a(i)+b(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 3 - previous step usage (read after write)
!-----------------------------------------------------------------
do i=1,n
   c(i) = c(i-1) + a(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 4 - next step usage (write after read)
!-----------------------------------------------------------------
do i=1,n
   c(i) = c(i+1) + a(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 5 - double assignment (write after write)
!-----------------------------------------------------------------
do i=1,n
   c(i) = a(i)
   c(i+1) = b(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 6 - reduction
!-----------------------------------------------------------------
summ=2._myk
print*,'summ: ',summ
summ = 0._myk
do i=1,n
   summ = summ + c(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 7 - loop bound with function
!-----------------------------------------------------------------
do i=1,whatis(n)
   c(i) = a(i)+b(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
print*, "what=",whatis(n)
!-----------------------------------------------------------------
! 8 - mixed data
!-----------------------------------------------------------------
do i=1,n
   c(i) = ai(i)+b(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 9 - branching in loop
!-----------------------------------------------------------------
do i=1,n
   if(a(i) .gt. 0.5_myk) then
     c(i) = 0.
   else
     c(i) = 1.
   endif
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 10 - branching - II
!-----------------------------------------------------------------
w = 2.73_myk
do i=1,n

  if( a(i) < 0 ) x = b(i)*w
  d(i) = a(i)+1
  if( a(i) < 0 ) c(i) = x*a(i)

!  if( a(i) < 0.5 ) then
!     x = b(i)*w
!     c(i) = x*a(i)
!  endif
!  d(i) = a(i)+1

enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 11 - mod operator inside loop
!-----------------------------------------------------------------
do i=1,n
   c(i) = mod(100*int(a(i)),7) + c(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 12 - complex index computation
!-----------------------------------------------------------------
do i=1,n
   ind = nint(sqrt(real(i,myk)))!+nint(log(real(i,myk)))
   c(ind) = mod(100*int(a(i)),7)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 13 - exit (break)
!-----------------------------------------------------------------
do i=1,n
   c(i) = a(i)-b(i)
   if(c(i) .lt. 0.) exit
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 14 - cycle (continue)
!-----------------------------------------------------------------
do i=1,n
   c(i) = a(i)-b(i)
   if(c(i) .lt. 0.) cycle
   c(i) = c(i)-2._myk*b(i)
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 15 - nested loops
!-----------------------------------------------------------------
do j=1,n
do i=1,n
   dc(i,j) = da(i,j)+db(i,j)
enddo
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 16 - nested loops II
!-----------------------------------------------------------------
do j=1,n
   da(i,j) = 0._myk
do i=1,n
   dc(i,j) = da(i,j)+db(i,j)
enddo
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 17 - calling function (try to inline it)
!-----------------------------------------------------------------
do i=1,n
   c(i) = double(c(i))
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
!-----------------------------------------------------------------
! 18 - calling mathematical function
!-----------------------------------------------------------------
do i=1,n
   c(i) = sin(c(i))
enddo
print*,'a(5),b(5),c(5),d(5): ',a(5),b(5),c(5),d(5)
print*,'da(5,5),db(5,5),dc(5,5): ',da(5,5),db(5,5),dc(5,5)

end program vectorization_test

function whatis(n)
integer :: whatis,n,a
real :: b
b = sin(n*0.43)
a=nint(b)
if(a+n.gt.10) then
   whatis = n*(a+nint(b))
else
   whatis = n*n*n
endif
return
end function whatis

function double(a)
integer, parameter :: myk=kind(1.d0)
real(myk) :: double,a
double=2.*a
return
end function double
